package com.hz.pm.api.scheduler.listener;

import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.helper.WflowHelper;
import com.hz.pm.api.external.MhApiClient;
import com.hz.pm.api.external.model.dto.MhZwddWorkNoticeDTO;
import com.hz.pm.api.external.todo.MHTodoClient;
import com.hz.pm.api.external.todo.dto.MhTodoExtraParamDTO;
import com.hz.pm.api.external.todo.enumerization.MHTodoTypeEnum;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectdeclared.model.entity.Xinchuang;
import com.hz.pm.api.projectdeclared.model.entity.XinchuangInst;
import com.hz.pm.api.projectdeclared.service.IPurchaseInstService;
import com.hz.pm.api.projectdeclared.service.IXinchuangInstService;
import com.hz.pm.api.projectdeclared.service.IXinchuangService;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.entity.WflowModelHistorys;
import com.wflow.workflow.notify.event.NodeCompleteEvent;
import com.wflow.workflow.notify.event.NodeCreateEvent;
import com.wflow.workflow.service.ProcessModelHistorysService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.flowable.task.api.history.HistoricTaskInstance;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.time.Instant;
import java.util.List;
import java.util.Set;

import static com.hz.pm.api.common.util.ThreadPoolUtil.SCHEDULER;

/**
 * <p>
 * TaskCreatedListener
 * </p>
 *
 * @author WendyYang
 * @since 10:38 2024/3/12
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class TaskCreatedListener {

    private final IProjectService projectService;
    private final IProjectInstService projectInstService;
    private final ProcessModelHistorysService processModelHistorysService;
    private final MHTodoClient mhTodoClient;
    private final MhApiClient mhApiClient;
    private final UserInfoHelper userInfoHelper;
    private final IPurchaseInstService purchaseInstService;
    private final IXinchuangInstService xinchuangInstService;
    private final IXinchuangService xinchuangService;
    private final TaskService taskService;

    private Instant instantDelay10s() {
        return Instant.now().plusSeconds(10);
    }

    @EventListener(classes = NodeCreateEvent.class)
    public void onEvent(NodeCreateEvent event) {
        SCHEDULER.schedule(() -> handleEvent(event), instantDelay10s());
    }

    @EventListener(classes = NodeCompleteEvent.class)
    public void onEvent(NodeCompleteEvent event) {
        SCHEDULER.schedule(() -> handleEvent(event), instantDelay10s());
    }

    public void handleEvent(NodeCreateEvent event) {
        String procDefId = event.getProcessDefinitionId();
        String procInstId = event.getInstCode();
        WflowModelHistorys models = processModelHistorysService.getByProcessDefId(procDefId);
        ProjectProcessType procType = ProjectProcessType.getNoNull(models.getProcessType());
        Task task = taskService.createTaskQuery()
                .taskId(event.getTaskId())
                .singleResult();
        UserFullInfoDTO assignee = userInfoHelper.getUserFullInfo(task.getAssignee());
        if (assignee == null || StrUtil.isBlank(assignee.getMhUserId())) {
            return;
        }
        switch (procType) {
            case XC_APPROVAL_PROCESS:
            case PROJECT_RECORD_APPROVAL_PROCESS:
            case ACCEPTANCE_DECLARATION_APPROVAL_PROCESS:
            case ADAPTION:
            case SELF_TEST:
            case TEST_VALID:
            case PROJECT_CHANGE:
            case PROJECT_STOPPED:
            case COMPLIANCE_REVIEW:
            case PROJECT_REVIEW:
                // 发送信产平台待办
                MHTodoTypeEnum todoType = getMhTodoTypeEnum(procType);
                MhTodoExtraParamDTO paramObj = MhTodoExtraParamDTO.builder()
                        .taskId(task.getId())
                        .instanceId(task.getProcessInstanceId())
                        .nodeId(task.getTaskDefinitionId())
                        .userId(assignee.getUserId())
                        .userName(assignee.getRealName())
                        .path("/toDoCenter/handleDuringExamine")
                        .build();
                if (procType.equals(ProjectProcessType.ACCEPTANCE_DECLARATION_APPROVAL_PROCESS)
                        || procType.equals(ProjectProcessType.ADAPTION)
                        || procType.equals(ProjectProcessType.XC_APPROVAL_PROCESS)
                        || procType.equals(ProjectProcessType.TEST_VALID)
                        || procType.equals(ProjectProcessType.SELF_TEST)) {
                    PurchaseInst purchaseInst = purchaseInstService.getByProjectIdAndInstCode(paramObj.getProjectId(), procInstId);
                    paramObj.setBidId(purchaseInst.getBidId());
                    paramObj.setProjectId(purchaseInst.getProjectId());
                }
                String projectName = "";
                // 上边查不到ID的时候说明是手动新增的信创项目
                if (paramObj.getBidId() == null) {
                    XinchuangInst xinchuangInst = xinchuangInstService.getByInstCode(procInstId);
                    if (xinchuangInst != null) {
                        Xinchuang xinchuang = xinchuangService.getById(xinchuangInst.getXinchuangId());
                        paramObj.setXinchuangId(xinchuang.getId());
                        projectName = xinchuang.getProjectName();
                    } else {
                        ProjectInst projectInst = projectInstService.getByInstCode(procInstId);
                        if (projectInst == null) {
                            return;
                        }
                        paramObj.setProjectId(projectInst.getProjectId());
                    }
                }
                if (paramObj.getProjectId() != null) {
                    Project project = projectService.getNewProject(paramObj.getProjectId());
                    projectName = project.getProjectName();
                    paramObj.setProjectId(project.getId());
                    paramObj.setProjectCode(project.getProjectCode());
                }
                String content = String.format("【%s】的【%s】需要您审核，请及时处理。", projectName, procType.getDesc());
                mhTodoClient.addTodo(paramObj, assignee, todoType, task.getId(), content);
                // 发送浙政钉工作通知
                if (assignee.getAccountId() != null) {
                    MhZwddWorkNoticeDTO notice = MhZwddWorkNoticeDTO.builder()
                            .content(content)
                            .title(todoType.getVal())
                            .targetUser(assignee.getMhUserId())
                            .build();
                    mhApiClient.sendZwddWorkNotice(notice);
                }
                break;
            default:
                break;
        }
    }

    public void handleEvent(NodeCompleteEvent event) {
        String procDefId = event.getProcessDefinitionId();
        WflowModelHistorys models = processModelHistorysService.getByProcessDefId(procDefId);
        ProjectProcessType procType = ProjectProcessType.getNoNull(models.getProcessType());
        WflowHelper wflowHelper = SpringUtil.getBean(WflowHelper.class);
        Task task = taskService.createTaskQuery()
                .taskId(event.getTaskId())
                .singleResult();
        boolean orUserTask = wflowHelper.isOrUserTask(procDefId, task.getTaskDefinitionKey());
        switch (procType) {
            case XC_APPROVAL_PROCESS:
            case PROJECT_RECORD_APPROVAL_PROCESS:
            case ACCEPTANCE_DECLARATION_APPROVAL_PROCESS:
            case ADAPTION:
            case SELF_TEST:
            case TEST_VALID:
            case PROJECT_CHANGE:
            case PROJECT_STOPPED:
            case COMPLIANCE_REVIEW:
            case PROJECT_REVIEW:
                MHTodoTypeEnum todoType = getMhTodoTypeEnum(procType);
                if (orUserTask) {
                    List<HistoricTaskInstance> tasks = wflowHelper.listFinishedTasks(task.getTaskDefinitionKey());
                    Set<String> taskIds = CollUtils.fieldSet(tasks, HistoricTaskInstance::getId);
                    taskIds.add(task.getId());
                    mhTodoClient.completeTodo(todoType, taskIds);
                } else {
                    mhTodoClient.completeTodo(todoType, task.getId());
                }
                break;
            default:
                break;
        }
    }

    private MHTodoTypeEnum getMhTodoTypeEnum(ProjectProcessType procType) {
        MHTodoTypeEnum todoType;
        switch (procType) {
            case XC_APPROVAL_PROCESS:
                todoType = MHTodoTypeEnum.XCFHX_INSPECT_AUDIT;
                break;
            case PROJECT_RECORD_APPROVAL_PROCESS:
                todoType = MHTodoTypeEnum.DECLARED_RECORD_AUDIT;
                break;
            case PROJECT_REVIEW:
                todoType = MHTodoTypeEnum.PROJECT_REVIEW_AUDIT;
                break;
            case COMPLIANCE_REVIEW:
                todoType = MHTodoTypeEnum.COMPLIANCE_REVIEW_AUDIT;
                break;
            case PROJECT_STOPPED:
                todoType = MHTodoTypeEnum.PROJECT_STOPPED_AUDIT;
                break;
            case TEST_VALID:
                todoType = MHTodoTypeEnum.TEST_VALID_AUDIT;
                break;
            case PROJECT_CHANGE:
                todoType = MHTodoTypeEnum.PROJECT_CHANGE_AUDIT;
                break;
            case ADAPTION:
                todoType = MHTodoTypeEnum.ADAPTION_AUDIT;
                break;
            case SELF_TEST:
                todoType = MHTodoTypeEnum.SELF_TEST_AUDIT;
                break;
            case ACCEPTANCE_DECLARATION_APPROVAL_PROCESS:
                todoType = MHTodoTypeEnum.FINAL_INSPECT_AUDIT;
                break;
            default:
                throw BizException.wrap("未知待办类型：" + procType);
        }
        return todoType;
    }

}
